"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Stream the value at a node.
 * @param {STStream} The state stream.
 * @param {string} id A string value.
 * @returns {Observable<Try<any>>} An Observable instance.
 */
exports.valueAtNode = function (stream, id) {
    return stream.map(function (v) { return v.valueAtNode(id); });
};
/**
 * Convenience method to stream a string from a node.
 * @param {STStream} The state stream.
 * @param {string} id A string value.
 * @returns {Observable<Try<string>>} An Observable instance.
 */
exports.stringAtNode = function (stream, id) {
    return stream.map(function (v) { return v.stringAtNode(id); });
};
/**
 * Convenience method to stream a number from a node.
 * @param {STStream} The state stream.
 * @param {string} id A string value.
 * @returns {Observable<Try<number>>} An Observable instance.
 */
exports.numberAtNode = function (stream, id) {
    return stream.map(function (v) { return v.numberAtNode(id); });
};
/**
 * Convenience method to stream a boolean from a node.
 * @param {STStream} The state stream.
 * @param {string} id A string value.
 * @returns {Observable<Try<boolean>>} An Observable instance.
 */
exports.booleanAtNode = function (stream, id) {
    return stream.map(function (v) { return v.booleanAtNode(id); });
};
/**
 * Convenience method to stream R from a node.
 * @template R Generics parameter.
 * @param {STStream} The state stream.
 * @param {new () => R} ctor Constructor function.
 * @param {string} id A string value.
 * @returns {Observable<Try<R>>} An Observable instance.
 */
function instanceAtNode(stream, ctor, id) {
    return stream.map(function (v) { return v.instanceAtNode(ctor, id); });
}
exports.instanceAtNode = instanceAtNode;
//# sourceMappingURL=util.js.map