"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rxjs_1 = require("rxjs");
var javascriptutilities_1 = require("javascriptutilities");
var src_1 = require("./../src");
var timeout = 100;
var path1 = 'a.b.c.d';
var path2 = 'a.b.c.d.e';
var path3 = 'a.b.c';
var actionKey1 = 'action1';
var actionKey2 = 'action2';
var actionKey3 = 'action3';
var numbers = [1, 2, 3, 4, 5];
var strings = ['1', '2', '3', '4', '5'];
var booleans = [true, false, true, false];
var testReduxStore = function (store, actionFn) {
    /// Setup
    var numbers = [1, 2, 3, 4, 5];
    var strings = ['1', '2', '3', '4', '5'];
    var booleans = [true, false, true, false];
    var values1 = [];
    var values2 = [];
    var values3 = [];
    var states = [];
    store.stateStream().doOnNext(function (v) { return states.push(v); }).subscribe();
    store.numberAtNode(path1).doOnNext(function (v) { return values1.push(v.value); }).subscribe();
    store.stringAtNode(path2).doOnNext(function (v) { return values2.push(v.value); }).subscribe();
    store.booleanAtNode(path3).doOnNext(function (v) { return values3.push(v.value); }).subscribe();
    /// When
    actionFn();
    /// Then
    expect(javascriptutilities_1.Collections.last(values1).value).toEqual(javascriptutilities_1.Numbers.sum(numbers));
    expect(javascriptutilities_1.Collections.last(values2).value).toEqual(strings.reduce(function (v1, v2) { return v1 + v2; }));
    expect(javascriptutilities_1.Collections.last(values3).value).toEqual(javascriptutilities_1.Collections.last(booleans).value);
    expect(states.every(function (v) { return v !== undefined && v !== null; })).toBeTruthy();
};
describe('Rx store should be implemented correctly', function () {
    var action1;
    var action2;
    var action3;
    var stateStore;
    var createStore = function () {
        var reducer1 = src_1.ReduxStore.Rx.createReducer(action1, function (state, v) {
            return state.mappingValue(path1, function (v1) {
                return v1.map(function (v2) { return v2 + v.value; }).successOrElse(javascriptutilities_1.Try.success(v.value));
            });
        });
        var reducer2 = src_1.ReduxStore.Rx.createReducer(action2, function (state, v) {
            return state.mappingValue(path2, function (v1) {
                return v1.map(function (v2) { return v2 + v.value; }).successOrElse(javascriptutilities_1.Try.success(v.value));
            });
        });
        var reducer3 = src_1.ReduxStore.Rx.createReducer(action3, function (state, v) {
            return state.updatingValue(path3, v.value);
        });
        return new src_1.ReduxStore.Rx.Self(reducer1, reducer2, reducer3);
    };
    beforeEach(function () {
        /// Here we mix both Action.Type and normal values.
        action1 = new rxjs_1.BehaviorSubject(0);
        action2 = new rxjs_1.BehaviorSubject({ name: actionKey2, value: '' });
        action3 = new rxjs_1.BehaviorSubject({ name: actionKey3, value: false });
        stateStore = createStore();
    });
    it('Dispatch action with subject - should work', function () {
        testReduxStore(stateStore, function () {
            numbers.forEach(function (v) { return action1.next(v); });
            setTimeout(undefined, timeout);
            strings.forEach(function (v) { return action2.next({ name: actionKey2, value: v }); });
            setTimeout(undefined, timeout);
            booleans.forEach(function (v) { return action3.next({ name: actionKey3, value: v }); });
            setTimeout(undefined, timeout);
        });
    });
});
describe('Dispatch store should be implemented correctly', function () {
    var stateStore;
    var actionFn1 = function (v) { return ({
        id: actionKey1,
        fullValuePath: path1,
        payload: v,
    }); };
    var actionFn2 = function (v) { return ({
        id: actionKey2,
        fullValuePath: path2,
        payload: v,
    }); };
    var actionFn3 = function (v) { return ({
        id: actionKey3,
        fullValuePath: path3,
        payload: v,
    }); };
    var reducer = function (state, action) {
        var payload = action.payload;
        switch (action.id) {
            case actionKey1:
                return state.mappingValue(action.fullValuePath, function (v) {
                    return v.map(function (v1) { return v1; }).getOrElse(0) + payload;
                });
            case actionKey2:
                return state.mappingValue(action.fullValuePath, function (v) {
                    return v.map(function (v1) { return v1; }).getOrElse('') + payload;
                });
            case actionKey3:
                return state.updatingValue(action.fullValuePath, action.payload);
            default:
                return state;
        }
    };
    beforeEach(function () {
        stateStore = src_1.ReduxStore.Dispatch.createDefault(reducer);
    });
    it('Dispatch action with action creators - should work', function () {
        testReduxStore(stateStore, function () {
            numbers.map(function (v) { return actionFn1(v); }).forEach(function (v) { return stateStore.dispatch(v); });
            setTimeout(undefined, timeout);
            strings.map(function (v) { return actionFn2(v); }).forEach(function (v) { return stateStore.dispatch(v); });
            setTimeout(undefined, timeout);
            booleans.map(function (v) { return actionFn3(v); }).forEach(function (v) { return stateStore.dispatch(v); });
            setTimeout(undefined, timeout);
        });
    });
});
//# sourceMappingURL=store.test.js.map